@@ -1,7 +1,7 @@
 //! Codemod
 use breezyshim::error::Error as BrzError;
-use breezyshim::tree::WorkingTree;
 use breezyshim::RevisionId;
+use breezyshim::WorkingTree;
 use std::collections::HashMap;
 use url::Url;
 
@@ -39,6 +39,23 @@ pub struct CommandResult {
     pub new_revision: RevisionId,
 }
 
+impl Default for CommandResult {
+    fn default() -> Self {
+        Self {
+            value: None,
+            context: None,
+            description: None,
+            serialized_context: None,
+            commit_message: None,
+            title: None,
+            tags: Vec::new(),
+            target_branch_url: None,
+            old_revision: RevisionId::null(),
+            new_revision: RevisionId::null(),
+        }
+    }
+}
+
 impl crate::CodemodResult for CommandResult {
     fn context(&self) -> serde_json::Value {
         self.context.clone().unwrap_or_default()
@@ -61,6 +78,26 @@ impl crate::CodemodResult for CommandResult {
     }
 }
 
+impl From<CommandResult> for DetailedSuccess {
+    fn from(r: CommandResult) -> Self {
+        DetailedSuccess {
+            value: r.value,
+            context: r.context,
+            description: r.description,
+            commit_message: r.commit_message,
+            title: r.title,
+            serialized_context: r.serialized_context,
+            tags: Some(
+                r.tags
+                    .into_iter()
+                    .map(|(k, v)| (k, v.map(|v| v.to_string())))
+                    .collect(),
+            ),
+            target_branch_url: r.target_branch_url,
+        }
+    }
+}
+
 impl From<&CommandResult> for DetailedSuccess {
     fn from(r: &CommandResult) -> Self {
         DetailedSuccess {
@@ -201,7 +238,7 @@ impl std::fmt::Display for DetailedFailure {
 /// - `script`: Script to run
 /// - `commit_pending`: Whether to commit pending changes
 pub fn script_runner(
-    local_tree: &WorkingTree,
+    local_tree: &dyn WorkingTree,
     script: &[&str],
     subpath: &std::path::Path,
     commit_pending: crate::CommitPending,
@@ -325,11 +362,12 @@ pub fn script_runner(
         if let Some(committer) = committer {
             builder = builder.committer(committer);
         }
-        new_revision = match builder.commit() {
-            Ok(rev) => rev,
+        match builder.commit() {
+            Ok(rev) => {
+                new_revision = rev;
+            }
             Err(BrzError::PointlessCommit) => {
-                // No changes
-                last_revision.clone()
+                // No changes - keep new_revision as last_revision
             }
             Err(e) => return Err(Error::Other(format!("Failed to commit changes: {}", e))),
         };
@@ -357,10 +395,103 @@ pub fn script_runner(
 }
 
 #[cfg(test)]
-mod script_runner_tests {
-    use breezyshim::tree::MutableTree;
+mod command_result_tests {
+    use super::*;
+    use crate::CodemodResult;
 
+    #[test]
+    fn test_command_result_context_with_value() {
+        let result = CommandResult {
+            context: Some(serde_json::json!({"key": "value"})),
+            ..Default::default()
+        };
+
+        assert_eq!(result.context(), serde_json::json!({"key": "value"}));
+    }
+
+    #[test]
+    fn test_command_result_context_none() {
+        let result = CommandResult {
+            context: None,
+            ..Default::default()
+        };
+
+        // Should return default (null) when context is None
+        assert_eq!(result.context(), serde_json::Value::Null);
+    }
+
+    #[test]
+    fn test_command_result_value() {
+        let result = CommandResult {
+            value: Some(42),
+            ..Default::default()
+        };
+
+        assert_eq!(result.value(), Some(42));
+    }
+
+    #[test]
+    fn test_command_result_description() {
+        let result = CommandResult {
+            description: Some("Test description".to_string()),
+            ..Default::default()
+        };
+
+        assert_eq!(result.description(), Some("Test description".to_string()));
+    }
+
+    #[test]
+    fn test_command_result_description_none() {
+        let result = CommandResult {
+            description: None,
+            ..Default::default()
+        };
+
+        assert_eq!(result.description(), None);
+    }
+
+    #[test]
+    fn test_command_result_target_branch_url() {
+        let url = url::Url::parse("https://github.com/test/repo").unwrap();
+        let result = CommandResult {
+            target_branch_url: Some(url.clone()),
+            ..Default::default()
+        };
+
+        assert_eq!(result.target_branch_url(), Some(url));
+    }
+
+    #[test]
+    fn test_command_result_tags() {
+        let tags = vec![
+            ("v1.0".to_string(), Some(RevisionId::from(b"rev1".to_vec()))),
+            ("v2.0".to_string(), None),
+        ];
+        let result = CommandResult {
+            tags: tags.clone(),
+            ..Default::default()
+        };
+
+        assert_eq!(result.tags(), tags);
+    }
+
+    #[test]
+    fn test_command_result_default() {
+        let result = CommandResult::default();
+
+        assert_eq!(result.context(), serde_json::Value::Null);
+        assert_eq!(result.value(), None);
+        assert_eq!(result.target_branch_url(), None);
+        assert_eq!(result.description(), None);
+        assert!(result.tags().is_empty());
+    }
+}
+
+#[cfg(test)]
+mod script_runner_tests {
     use breezyshim::controldir::create_standalone_workingtree;
+    use breezyshim::tree::MutableTree;
+    use breezyshim::WorkingTree;
 
     fn make_executable(script_path: &std::path::Path) {
         #[cfg(unix)]
